/** @file qt_screenmode.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2019.10.01

	@brief [ screen mode ]
*/

#include "qt_screenmode.h"
#include <QApplication>
#include <QScreen>
#include "../../emu.h"
#include "../../main.h"


ScreenMode::ScreenMode()
{
	disp_device_count = 0;
	monitor_count = 0;
	screen_mode_count = 0;
}

ScreenMode::~ScreenMode()
{
}

void ScreenMode::Enum(int desktop_width, int desktop_height, uint32_t UNUSED_PARAM(flags))
{
	// enumerate display device
	int max_desktop_width = 0;
	int max_desktop_height = 0;

	QList<QScreen *> scrns = qApp->screens();
	disp_device_count = scrns.count();
	if (disp_device_count > DISP_DEVICE_MAX) {
		disp_device_count = DISP_DEVICE_MAX;
	}

	for(int disp_no = 0; disp_no < disp_device_count; disp_no++) {
		QSize sz = scrns[disp_no]->size();
		disp_device[disp_no].re.x = 0;
		disp_device[disp_no].re.w = 0;
		disp_device[disp_no].re.w = sz.width();
		disp_device[disp_no].re.h = sz.height();
	}

	// enumerate screen mode for fullscreen
	int w,h,bpp;
	screen_mode_count = 0;

	for(int disp_no = 0; disp_no < disp_device_count && screen_mode_count < SCREEN_MODE_MAX; disp_no++) {
		QSize sz = scrns[disp_no]->availableSize();

		for(int i = 0; i < 1 && screen_mode_count < SCREEN_MODE_MAX; i++) {
			w = sz.width();
			h = sz.height();
			bpp = 0;
			if(w < MIN_WINDOW_WIDTH || h < MIN_WINDOW_HEIGHT || w < 640 || h < 480) {
				logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp ignored"), disp_no, w, h, bpp);
				continue;
			}
			if(((w * 30 / h) != 40)
			&& ((w * 40 / h) != 50)
			&& ((w * 90 / h) != 160)
			&& ((w * 50 / h) != 80)) {
				logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp ignored"), disp_no, w, h, bpp);
				continue;
			}
			if (w <= 1280 && (w != 640 && w != 800 && w != 1024 && w != 1280)) {
				logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp ignored"), disp_no, w, h, bpp);
				continue;
			}
			bool found = false;
			for(int j = 0; j < screen_mode_count; j++) {
				if(screen_mode[j].disp_no == disp_no && screen_mode[j].width == w && screen_mode[j].height == h) {
					found = true;
					logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp already exist"), disp_no, w, h, bpp);
					break;
				}
			}
			if(!found) {
				screen_mode[screen_mode_count].disp_no = disp_no;
				screen_mode[screen_mode_count].width = w;
				screen_mode[screen_mode_count].height = h;
				logging->out_debugf(_T("screen_mode:%2d [%d] %dx%d %dbpp"), screen_mode_count, disp_no, w, h, bpp);
				screen_mode_count++;

				if(max_desktop_width <= w) {
					max_desktop_width = w;
				}
				if(max_desktop_height <= h) {
					max_desktop_height = h;
				}
			}
		}
	}
	qsort(&screen_mode, static_cast<size_t>(screen_mode_count), sizeof(struct screen_mode_st), qsort_by_n_w_h);

	// if cannot get modes, add default screen size mode.
	if (screen_mode_count == 0) {
		screen_mode[screen_mode_count].disp_no = 0;
		screen_mode[screen_mode_count].width = desktop_width;
		screen_mode[screen_mode_count].height = desktop_height;
		screen_mode_count = 1;

		max_desktop_width = desktop_width;
		max_desktop_height = desktop_height;
	}
}

int ScreenMode::qsort_by_n_w_h(const void *a, const void *b)
{
	const struct screen_mode_st *a1 = reinterpret_cast<const struct screen_mode_st *>(a);
	const struct screen_mode_st *b1 = reinterpret_cast<const struct screen_mode_st *>(b);

	int rc = 0;
	int aa = a1->width * a1->height;
	int bb = b1->width * b1->height;
	if (!rc) rc = (a1->disp_no - b1->disp_no);
	if (!rc) rc = (bb - aa);
	return rc;
}

const screen_mode_t *ScreenMode::GetMode(int num) const
{
	if (0 <= num && num < screen_mode_count) {
		return &screen_mode[num];
	}
	return nullptr;
}

int ScreenMode::FindMode(int disp_no, int width, int height) const
{
	int match = -1;
	for(int i = 0; i < screen_mode_count; i++) {
		if (screen_mode[i].disp_no == disp_no
		&&  screen_mode[i].width == width
		&&  screen_mode[i].height == height) {
			match = i;
			break;
		}
	}
	return match;
}

const disp_device_t *ScreenMode::GetDisp(int num) const
{
	if (0 <= num && num < disp_device_count) {
		return &disp_device[num];
	}
	return nullptr;
}
