/** @file qt_socket.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2017.03.01

	@brief [ qt socket ]
*/

#ifndef QT_SOCKET_H
#define QT_SOCKET_H

#include "../../common.h"
#include <QAbstractSocket>
#include <QTcpSocket>
#include <QUdpSocket>
#include <QTcpServer>
#include <QHostInfo>
#include <QTimer>

class EMU;
class MySocket;

/**
 * network
 */
class MyConnection : public QObject
{
	Q_OBJECT

private:
	EMU *emu;
	MySocket **mSocket;
	bool *mSocketUsed;
	int mSocketMax;
	int mSocketNum;

public:
	MyConnection(EMU *parent, int max_sockets);
	~MyConnection();
	bool Connect(int new_ch, const _TCHAR *hostname, int port, bool server = false, bool tcp = true);
	bool Connect(const _TCHAR *hostname, int port, bool server = false, bool tcp = true);
	bool AcceptSocket(int ch, MySocket *server, QAbstractSocket *sock, bool tcp);
	int  GetSocketChannel();
	void Disconnect(int ch);
	size_t Send(int ch, const char *buffer, size_t len);
	size_t Recv(int ch, char *buffer, size_t len);
	size_t SendTo(int ch, const _TCHAR *hostname, int port, const void *buffer, size_t len);
	size_t RecvFrom(int ch, char *ipaddr, int *port, void *buffer, size_t len);

	void OnDestroy(int ch);

	EMU *GetEMU() { return emu; }
};

class MySocket : public QObject
{
	Q_OBJECT

private:
	MyConnection *conn;
	EMU *emu;
	int m_ch;
	QAbstractSocket *mSocket;
	QTcpServer *mServer;
	bool m_ok;
	bool m_tcp;
	bool m_isServer;
	MySocket *mConnServer;
	int m_numClient;

	QTimer mTimer;
	int mTimerType;

//	bool m_busy;
	void IncreaseClient();
	void DecreaseClient();
	int numClient();

public:
	MySocket(MyConnection *parent, int ch, const _TCHAR *hostname, int port, bool server = false, bool tcp = true);
//	MySocket(MyConnection *parent, int ch, const QString &hostname, const wxString &service, bool server);
	MySocket(MyConnection *parent, MySocket *server, int ch, QAbstractSocket *sock);
	~MySocket();

	// slots
public slots:
	void onAcceptSlot();
	void onConnectSlot();
//	void OnSocketEvent(wxSocketEvent &);
	void onReadSlot();
//	void OnSocketLostEvent(wxSocketEvent &);
	void onTimerSlot();

	// properties
public:
	bool IsOk() { return m_ok; }

	bool Connect(const _TCHAR *hostname, int port);
//	bool Connect(const QString &hostname, const wxString &service);
	void Disconnect();
	size_t Send(const char *buffer, size_t len);
	size_t Recv(char *buffer, size_t len);
	size_t SendTo(const _TCHAR *hostname, int port, const void *buffer, size_t len);
	size_t RecvFrom(char *ipaddr, int *port, void *buffer, size_t len);
};

#endif /* QT_SOCKET_H */
